/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mouse Gestures for Mozilla.
 *
 * The Initial Developer of the Original Code is Jens Bannmann.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s) (alphabetical order):
 *  Jens Bannmann <jens.b@web.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var treeDS;
var mappingsTree;
var typesTree;
var dmTree;
var mappingsContainer;
var defContainer;
var curType;
var typePrefix = new Array();
var bundle;

var mCodeCol;
var dmCodeCol;

function initMappingsWindow() {
  // get all elements we'll work with
  bundle = document.getElementById("mozgestbundle");
  mappingsTree = document.getElementById("mappingstree");
  typesTree = document.getElementById("typesTree");
  dmTree = document.getElementById("defaultmappingstree");
  mCodeCol = document.getElementById("codeCol");
  dmCodeCol = document.getElementById("dmCodeCol");

  treeDS = new mgMappingsDS();
  mappingsTree.database.AddDataSource(treeDS);

  var tChildren = document.getElementById("typestreechildren");
  var dsTypes = mgStorage.RDF.GetDataSourceBlocking("chrome://mozgest/content/windowTypes.rdf");
  typesList = Components.classes['@mozilla.org/rdf/container;1']
                     .createInstance()
                     .QueryInterface(Components.interfaces.nsIRDFContainer);
  try {
    typesList.Init(dsTypes, mgStorage.RDF.GetResource("urn:mozgest:windows:list"));
  } catch (e) {
    dump("MozGest: Could not init typesList. " + e + "\n");
  }
  var typesEnum = typesList.GetElements();
  while (typesEnum.hasMoreElements()) {
    var type = typesEnum.getNext()
               .QueryInterface(Components.interfaces.nsIRDFResource);
    if (!mgStorage.getProperty(dsTypes, type, "separator")) {
      var typeCode = mgStorage.getProperty(dsTypes, type, "wtype");
      addTreeRow(tChildren,
                 new Array(mgStorage.getProperty(dsTypes, type, "label"),
                           typeCode));
      typePrefix[typeCode] = mgStorage.getProperty(dsTypes, type, "prefix");
    } else {
      tChildren.appendChild(document.createElement("treeseparator"));
    }
  }
  typesTree.contentView.selection.select(0);
  typesTree.focus();
  showMappingsFor("window");

  typesTree.addEventListener("select", showSelectedComponent, false);

  mCodeCol.addEventListener("DOMAttrModified", codeColChanged, true);
}


// synchronize column widths
function codeColChanged(event) {
  if (event.target == mCodeCol && event.attrName == "width") {
    dmCodeCol.setAttribute("width", event.newValue);
  }
}

function showSelectedComponent() {
  var row = typesTree.contentView.selection.currentIndex;
  if (!typesTree.treeBoxObject.view.isSeparator(row)) {
    showMappingsFor(typesTree.treeBoxObject.view.getCellText(row, "typeCol"));
  }
}

function showMappingsFor(component) {
  mappingsTree.ref = "urn:mozgest:mappings:" + component;
  mappingsTree.builder.rebuild();

  var dsDefault = mgStorage.getDefaultMappings();
  mappingsContainer = mgStorage.newContainer();
  defContainer = mgStorage.newContainer();
  var containerNode = mgStorage.RDF.GetResource(mappingsTree.ref);
  try {
    mappingsContainer.Init(treeDS, containerNode);
  } catch (e) {
    // create mappings container in treeDS...
    mgStorage.RDFCUtils.MakeSeq(treeDS, containerNode);
    mappingsContainer.Init(treeDS, containerNode);
    // ... and in user store to make sure treeDS can copy mappings back
    mgStorage.RDFCUtils.MakeSeq(mgStorage.getUserRDF(), containerNode);
    mgStorage.newContainer().Init(mgStorage.getUserRDF(), containerNode);
  }
  try {
    defContainer.Init(dsDefault, containerNode);
  } catch (e) {
    alert("MozGest: Fatal error - could not init defaultMappingsContainer.\n"
          + e);
  }
  showRemovedDefaults();

  curType = component;
}

function showRemovedDefaults() {
  var dsDefault = mgStorage.getDefaultMappings();

  // determine which mappings are (will be) installed
  var installed = new Array();
  var mappingsEnum = mappingsContainer.GetElements();
  while (mappingsEnum.hasMoreElements()) {
    var mapping = mappingsEnum.getNext();
    var mCode = mgStorage.getProperty(treeDS, mapping, "code");
    var mFunction = mgStorage.getProperty(treeDS, mapping, "function");
    
    // default mappings have non-localized codes, so we must deLocalize here
    installed[mgMappingLocalizer.deLocalize(mCode) + "~" + mFunction] = true;
  }

  // clear default mapping tree
  tChildren = document.getElementById("defMappingRows");
  while(tChildren.firstChild) {
    tChildren.removeChild(tChildren.firstChild);
  }

  // determine default mappings that will be missing & display them
  var defaultsEnum = defContainer.GetElements();
  while (defaultsEnum.hasMoreElements()) {
    var mapping = defaultsEnum.getNext()
                  .QueryInterface(Components.interfaces.nsIRDFResource);
    var mCode = mgStorage.getProperty(dsDefault, mapping, "code");
    var mFunction = mgStorage.getProperty(dsDefault, mapping, "function");
    if (!installed[mCode + "~" + mFunction]) {
      var funcName = mgStorage.getProperty(dsDefault, mapping, "function");
      addTreeRow(tChildren, new Array(mgMappingLocalizer.localize(mCode),
                                  bundle.getString(funcName),
                                  mapping.Value));
    }
  }
}


/*** mapping actions ***/

function newMapping() {
  openEditMappingWindow("", "", "", "", "", "");
}

function editMapping(passURI) {
  var row = mappingsTree.currentIndex;
  if (row < 0) {
    return;
  }
  var node = mappingsTree.builderView.getResourceAtIndex(row);
  openEditMappingWindow((passURI) ? node.Value : "",
                        mgStorage.getProperty(treeDS, node, "code"),
                        mgStorage.getProperty(treeDS, node, "appearance"),
                        mgStorage.getProperty(treeDS, node, "function"),
                        mgStorage.getProperty(treeDS, node, "name"),
                        decodeURI(mgStorage.getProperty(treeDS, node, "custom")));
}

function openEditMappingWindow(uri, code, appearance, func, name, custom) {
  window.openDialog("chrome://mozgest/content/pref/edit-mapping.xul",
                    "mozgest:editmapping", "chrome,centerscreen,modal=yes",
                    uri, code, appearance, func, name, custom,
                    typePrefix[curType], curType);
}

function receiveEditedMapping(uri, code, appearance, func, name, custom) {
  if (!duplicateCheck(code, mappingsContainer, uri)) {
    return false;
  }
  if (uri != "") {
    removeMappingNode(mgStorage.RDF.GetResource(uri));
  }

  node = mgStorage.RDF.GetAnonymousResource();
  mgStorage.assertLiteral(treeDS, node, "code", code);
  mgStorage.assertLiteral(treeDS, node, "name", name);
  if (func != "") {
    mgStorage.assertLiteral(treeDS, node, "function", func);
  }
  if (appearance != "") {
    mgStorage.assertLiteral(treeDS, node, "appearance", appearance);
  }
  if (custom != "") {
    mgStorage.assertLiteral(treeDS, node, "custom", encodeURI(custom));
  }
  mappingsContainer.AppendElement(node, true);
  treeDS.mindAdded(node);

  showRemovedDefaults();
  return true;
}

function removeMapping() {
  var row = mappingsTree.currentIndex;
  if (row < 0) {
    return;
  }
  removeMappingNode(mappingsTree.builderView.getResourceAtIndex(row));
  showRemovedDefaults();
}


/*** commands for default mappings ***/

function addDefault() {
  // determine RDF node (get URN from dmResCol column)
  var row = dmTree.contentView.selection.currentIndex;
  if (row < 0) {
    return false;
  }
  if (!duplicateCheck(dmTree.treeBoxObject.view.getCellText(row, "dmCodeCol"),
                      mappingsContainer, "")) {
    return false;
  }
  var urn = dmTree.treeBoxObject.view.getCellText(row, "dmResCol");
  var sourceNode = mgStorage.RDF.GetResource(urn);

  // read the mapping's data
  var dsDefault = mgStorage.getDefaultMappings();
  var mCode = mgStorage.getProperty(dsDefault, sourceNode, "code");
  var mFunction = mgStorage.getProperty(dsDefault, sourceNode, "function");
  var mAppearance = mgStorage.getProperty(dsDefault, sourceNode, "appearance");

  // create new node for treeDS (with localized data)
  var newNode = mgStorage.RDF.GetAnonymousResource();
  mgStorage.assertLiteral(treeDS, newNode, "code",
                          mgMappingLocalizer.localize(mCode));
  if (mAppearance) {
    mgStorage.assertLiteral(treeDS, newNode, "appearance", 
                       mgMappingLocalizer.localize(mAppearance));
  }
  mgStorage.assertLiteral(treeDS, newNode, "function", mFunction);
  mgStorage.assertLiteral(treeDS, newNode, "name",
                          mgMappingLocalizer.getString(mFunction));
  mappingsContainer.AppendElement(newNode, true);

  treeDS.mindAdded(newNode);
  showRemovedDefaults();
}


/*** common mapping storage functions ***/

function duplicateCheck(code, targetContainer, uri) {
  var res = mgStorage.findMapping(mgMappingLocalizer.deLocalize(code),
                                  targetContainer, treeDS);
  if (res) {
    if (uri != res.Value) {
      var msg = bundle.getString("overwriteWarning");
      var code = mgStorage.getProperty(treeDS, res, "code");
      msg = msg.replace(/%MAPPINGNAME%/,
                        mgStorage.getProperty(treeDS, res, "name"));
      msg = msg.replace(/%CODE%/, code);
      if (!confirm(msg)) {
        return false;
      } else {
        removeMappingNode(res);
      }
    }
  }
  return true;
}

function removeMappingNode(node) {
  mappingsContainer.RemoveElement(node, true);
  var arcs = treeDS.ArcLabelsOut(node);
  while(arcs.hasMoreElements()) {
    var arc = arcs.getNext();
    var targets = treeDS.GetTargets(node, arc, true);
    while (targets.hasMoreElements()) {
      var target = targets.getNext();
      treeDS.Unassert(node, arc, target, true);
    }
  }
  treeDS.mindDeleted(node);
}

function onAcceptMappings() {
  treeDS.applyChanges();
  return true;
}
