// Popup Count 0.2.5
// By Pike

var PopupCount = {

	FileFlags: {
		// From http://lxr.mozilla.org/mozilla/source/nsprpub/pr/include/prio.h#600
		RDONLY: 0x01,
		WRONLY: 0x02,
		RDWR: 0x04,
		CREATE_FILE: 0x08,
		APPEND: 0x10,
		TRUNCATE: 0x20,
		SYNC: 0x40,
		EXCL: 0x80
	},
	
	prefService: null,
	saveFile: null,
	saveFileMode: 660,
	localeStrings: null,
	originalTooltip: null,
	totalBlocked: 0,
	
	onLoad: function()
	{
		// Setup object properties
		this.prefService = Components.classes['@mozilla.org/preferences-service;1'].getService(Components.interfaces.nsIPrefService).getBranch('popupcount.');
		var dirService = Components.classes['@mozilla.org/file/directory_service;1'].getService(Components.interfaces.nsIProperties);
		var saveFileName = dirService.get('ProfD', Components.interfaces.nsILocalFile);
		saveFileName.append('popupcount.dat');
		this.saveFile = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
		this.saveFile.initWithFile(saveFileName);
		this.localeStrings = document.getElementById('popup-count-strings');
		this.originalTooltip = document.getElementById('page-report-button').getAttribute('tooltiptext');
		
		// Setup event listeners
		window.addEventListener('unload', function(event) {PopupCount.onUnload(event);}, true);
		window.addEventListener('DOMUpdatePageReport', function() {PopupCount.onUpdatePageReport();}, false);
		window.addEventListener('DOMPopupBlocked', function() {PopupCount.onBlockedPopup();}, false);
		
		// Load total blocked from profile
		if (this.saveFile.exists())
		{
			var aLocalStream = Components.classes['@mozilla.org/network/file-input-stream;1'].createInstance(Components.interfaces.nsIFileInputStream);
			aLocalStream.init(this.saveFile, this.FileFlags.RDONLY, this.saveFileMode, 0);
			var aLocalScriptStream = Components.classes['@mozilla.org/scriptableinputstream;1'].createInstance(Components.interfaces.nsIScriptableInputStream);
			aLocalScriptStream.init(aLocalStream);
		    this.totalBlocked = aLocalScriptStream.read(aLocalScriptStream.available());
			aLocalStream.close();
		}
	},
	
	onUnload: function(event)
	{
		// Save total blocked to profile
		var aLocalStream = Components.classes['@mozilla.org/network/file-output-stream;1'].createInstance(Components.interfaces.nsIFileOutputStream);
		aLocalStream.init(this.saveFile, this.FileFlags.WRONLY | this.FileFlags.CREATE_FILE | this.FileFlags.TRUNCATE, this.saveFileMode, 0);
		var output = new String(this.totalBlocked);
		aLocalStream.write(output, output.length);
		aLocalStream.close();
	},
	
	onUpdatePageReport: function()
	{
		if (getBrowser().mCurrentBrowser.pageReport)
		{
			window.setTimeout('PopupCount.displayBlocked()', 100); /*** Find better alternative using only DOMPopupBlocked ***/
		}
	},
	
	onBlockedPopup: function()
	{
		this.totalBlocked++;
		this.displayBlocked();
	},
	
	displayBlocked: function()
	{
		var output = getBrowser().mCurrentBrowser.pageReport.length;
		var tooltipText = true;
		try
		{
			tooltipText = this.prefService.getBoolPref('total_as_tooltip');
		}
		catch (err) {}
		if (tooltipText == true)
		{
			tooltipText = this.localeStrings.getString('totalPopupsBlocked') + ': ' + this.totalBlocked;
		}
		else
		{
			output = output + ' (' + this.totalBlocked + ')';
			tooltipText = this.originalTooltip;
		}
		document.getElementById('page-report-count').value = output;
		document.getElementById('page-report-button').setAttribute('tooltiptext', tooltipText);
	}
};

window.addEventListener('load', function() {PopupCount.onLoad();}, false);

