// Search Button Lite 0.2
// By Pike

const SearchButtonLiteObserver = {

	observe: function(prefService, topic, pref)
	{
		SearchButtonLite.checkPrefs(prefService);
	}
};

var SearchButtonLite = {

	ButtonActions: {
		Nothing: 0,
		SearchCurrent: 1,
		SearchNewFg: 2,
		SearchNewBg: 3,
		ClearSearchBar: 4,
		EngineMenu: 5
	},
	
	prefService: null,
	buttonPref: [5, 2, 1],
	enterPref: 1,
	autoClearPref: false,
	
	onLoad: function()
	{
		// Setup object properties
		this.prefService = Components.classes['@mozilla.org/preferences-service;1'].getService(Components.interfaces.nsIPrefService).getBranch('searchbuttonlite.');
		this.checkPrefs(this.prefService);
		
		// Set up preference observer (allowing interface update from settings dialog)
		var prefBranch = this.prefService.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
		if (prefBranch) prefBranch.addObserver('', SearchButtonLiteObserver, false);
	},
	
	onClick: function(event)
	{
		this.doSearch(this.buttonPref[event.button], event);
	},
	
	onEnter: function(event)
	{
		this.doSearch(this.enterPref, event);
	},
	
	doSearch: function(pref, event)
	{
		switch (pref)
		{
			case this.ButtonActions.SearchCurrent:
				this.handleSearchBarCommand(event, event.altKey ? true : false, event.ctrlKey ? false : true);
				break;
			case this.ButtonActions.SearchNewFg:
				this.handleSearchBarCommand(event, event.altKey ? false : true, event.ctrlKey ? false : true);
				break;
			case this.ButtonActions.SearchNewBg:
				this.handleSearchBarCommand(event, event.altKey ? false : true, event.ctrlKey ? true : false);
				break;
			case this.ButtonActions.ClearSearchBar:
				this.clearBar();
				break;
		}
	},
	
	checkPrefs: function(prefService)
	{
		try
		{
			this.buttonPref[0] = prefService.getIntPref('left_button');
		}
		catch (err) {}
		try
		{
			this.buttonPref[1] = prefService.getIntPref('middle_button');
		}
		catch (err) {}
		try
		{
			this.buttonPref[2] = prefService.getIntPref('right_button');
		}
		catch (err) {}
		try
		{
			this.enterPref = prefService.getIntPref('enter_key');
		}
		catch (err) {}
		try
		{
			this.autoClearPref = prefService.getBoolPref('auto_clear');
		}
		catch (err) {}
		this.addMenu();
	},
	
	addMenu: function()
	// Add engine menu
	{
		var searchButton = document.getElementById('search-proxy-button');
		
		// Add/remove menu from left mouse button
		var popup = searchButton.getAttribute('popup');
		if (this.buttonPref[0] == this.ButtonActions.EngineMenu)
		{
			if (popup != 'SearchBarPopup') searchButton.setAttribute('popup', 'SearchBarPopup');
		}
		else
		{
			if (popup) searchButton.removeAttribute('popup');
		}
		
		// Add/remove menu from right mouse button
		var context = searchButton.getAttribute('context');
		if (this.buttonPref[2] == this.ButtonActions.EngineMenu)
		{
			if (context != 'SearchBarPopup') searchButton.setAttribute('context', 'SearchBarPopup');
		}
		else
		{
			if (context) searchButton.removeAttribute('context');
		}
	},
	
	searchPopup: function(event)
	{
		if (event.target.id != 'search-button-lite-clear')
		{
			SearchBarPopupCommand(event);
		}
	},
	
	openEngines: function()
	{
		document.getElementById('SearchBarPopup').showPopup(document.getElementById('search-proxy-button'), -1, -1, 'popup', 'bottomleft', 'topleft');
	},
	
	clearBar: function()
	{
		document.getElementById('search-bar').value = '';
		focusSearchBar();
	},
	
	handleSearchBarCommand: function(aEvent, openInTab, bringToFront)
	// Modified version of handleSearchBarCommand() from browser.js and onTextEntered() from search.xml
	{
		var searchBar = document.getElementById('search-bar');
		
		// Save the current value in the form history
		if (gFormFillEnabled)
		{
			if (!gFormHistory)
			{
				gFormHistory = Components.classes['@mozilla.org/satchel/form-history;1'].getService(Components.interfaces.nsIFormHistory);
			}
			gFormHistory.addEntry(searchBar.getAttribute('autocompletesearchparam'), searchBar.value);
		}
		
		if (this.isSearchEngine())
		{
			var searchUrl = this.getSearchUrl();
			if (openInTab)
			{
				var newTab = getBrowser().addTab(searchUrl);
				if (bringToFront) getBrowser().selectedTab = newTab;
			}
			else
			{
				loadURI(searchUrl);
			}
			if (this.autoClearPref) this.clearBar();
		}
		else
		{
			quickFindInPage(searchBar.value);
		}
	},
	
	isSearchEngine: function()
	{
		var searchBar = document.getElementById('search-bar');
		if (window.navigator.productSub && window.navigator.productSub < 20030807)
		// browser.js version 1.216 or less
		{
			return (searchBar.hasAttribute('searchmode'));
		}
		else if (window.navigator.productSub && window.navigator.productSub < 20031022)
		// browser.js version 1.217 to 1.240
		{
			return (searchBar.getAttribute('searchmode') != '__PhoenixFindInPage');
		}
		else
		// browser.js version 1.241 and above
		{
			return (searchBar.getAttribute('searchengine') != '__PhoenixFindInPage');
		}
	},
	
	getSearchUrl: function()
	{
		var searchBar = document.getElementById('search-bar');
		if (window.navigator.productSub && window.navigator.productSub < 20031022)
		// browser.js version 1.240 and below
		{
			return searchBar.searchValue;
		}
		else
		// browser.js version 1.241 and above
		{
			var iSearchService = Components.classes['@mozilla.org/rdf/datasource;1?name=internetsearch'].getService(Components.interfaces.nsIInternetSearchService);
			return iSearchService.GetInternetSearchURL(searchBar.getAttribute('searchengine'), window.escape(searchBar.value), 0, 0, {value: 0});
		}
	}
};

window.addEventListener('load', function() {SearchButtonLite.onLoad();}, false);

